package ru.dbotthepony.mc.otm.android.feature

import net.minecraft.nbt.CompoundTag
import net.minecraft.server.level.ServerPlayer
import net.minecraftforge.event.entity.living.LivingHurtEvent
import ru.dbotthepony.mc.otm.android.AndroidFeature
import ru.dbotthepony.mc.otm.capability.MatteryPlayerCapability
import ru.dbotthepony.mc.otm.capability.extractEnergyInnerExact
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.registry.AndroidFeatures
import ru.dbotthepony.mc.otm.registry.StatNames
import ru.dbotthepony.mc.otm.container.set
import ru.dbotthepony.mc.otm.core.set
import kotlin.math.roundToInt

class NanobotsArmor(android: MatteryPlayerCapability) : AndroidFeature(AndroidFeatures.NANOBOTS_ARMOR, android) {
	var strength: Int = 0
		set(value) { field = value.coerceIn(0 .. 2) }

	var speed: Int = 0
		set(value) { field = value.coerceIn(0 .. 2) }

	private var ticksPassed = 0
	private var layers = 0

	override fun tickServer() {
		if (layers < strength + 1 && android.extractEnergyInnerExact(ENERGY_PER_LAYER, true).isPositive) {
			ticksPassed++

			if (ticksPassed >= TICKS[speed]) {
				layers++
				android.extractEnergyInner(ENERGY_PER_LAYER, false)
			}
		} else {
			ticksPassed = 0
		}
	}

	override fun onHurt(event: LivingHurtEvent) {
		ticksPassed = 0

		if (!event.source.isBypassArmor && layers > 0) {
			val absorbed = event.amount * STRENGTH[layers]

			if (absorbed > 0.1f) {
				val powerRequired = ENERGY_PER_HITPOINT * absorbed
				val powerExtracted = android.extractEnergyInner(powerRequired, false)
				val realAbsorbed = (powerExtracted / ENERGY_PER_HITPOINT).toFloat()
				event.amount = event.amount - realAbsorbed
				(entity as ServerPlayer?)?.awardStat(StatNames.DAMAGE_ABSORBED, (realAbsorbed * 10f).roundToInt())
				layers--
			}
		}
	}

	override fun serializeNBT(): CompoundTag {
		return super.serializeNBT().also {
			it["ticksPassed"] = ticksPassed
			it["layers"] = layers
			it["strength"] = strength
			it["speed"] = speed
		}
	}

	override fun deserializeNBT(nbt: CompoundTag) {
		super.deserializeNBT(nbt)
		ticksPassed = nbt.getInt("ticksPassed")
		layers = nbt.getInt("layers")
		strength = nbt.getInt("strength")
		speed = nbt.getInt("speed")
	}

	companion object {
		private val ENERGY_PER_LAYER = ImpreciseFraction(200)
		private val ENERGY_PER_HITPOINT = ImpreciseFraction(500)

		private val TICKS = listOf(
			80, // 4 seconds to build a layer
			70, // 3.5 seconds to build a layer
			60, // 3 seconds to build a layer
			50, // 2.5 seconds to build a layer
		)

		private val STRENGTH = listOf(
			0.1f,
			0.2f,
			0.3f,
			0.4f,
		)
	}
}
