package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.core.BlockPos
import net.minecraft.network.chat.Component
import net.minecraft.server.level.ServerPlayer
import net.minecraft.world.MenuProvider
import net.minecraft.world.entity.LivingEntity
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.phys.AABB
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.WorkerEnergyStorage
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.core.ifPresentK
import ru.dbotthepony.mc.otm.menu.AndroidStationMenu
import ru.dbotthepony.mc.otm.registry.MBlockEntities

class AndroidStationBlockEntity(p_155229_: BlockPos, p_155230_: BlockState) :
	MatteryPoweredBlockEntity(MBlockEntities.ANDROID_STATION, p_155229_, p_155230_), MenuProvider {

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return AndroidStationMenu(containerID, inventory, this)
	}

	override val defaultDisplayName: Component
		get() = MACHINE_NAME

	override val energy = object : WorkerEnergyStorage(this@AndroidStationBlockEntity, STORAGE, MAX_IO, null) {
		override fun extractEnergyInner(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
			return super.extractEnergyInner(howMuch, simulate).also {
				if (!simulate && this.batteryLevel.isZero) {
					if (blockState.getValue(WorkerState.SEMI_WORKER_STATE) != WorkerState.IDLE) {
						level?.setBlock(blockPos, blockState.setValue(WorkerState.SEMI_WORKER_STATE, WorkerState.IDLE), Block.UPDATE_CLIENTS)
					}
				}
			}
		}

		override fun receiveEnergyInner(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
			return super.receiveEnergyInner(howMuch, simulate).also {
				if (!simulate && it.isPositive) {
					if (blockState.getValue(WorkerState.SEMI_WORKER_STATE) != WorkerState.WORKING) {
						level?.setBlock(blockPos, blockState.setValue(WorkerState.SEMI_WORKER_STATE, WorkerState.WORKING), Block.UPDATE_CLIENTS)
					}
				}
			}
		}
	}

	private var tickedOnve = false

	fun tick() {
		if (!tickedOnve) {
			tickedOnve = true

			if (energy.batteryLevel.isPositive && blockState.getValue(WorkerState.SEMI_WORKER_STATE) != WorkerState.WORKING) {
				level?.setBlock(blockPos, blockState.setValue(WorkerState.SEMI_WORKER_STATE, WorkerState.WORKING), Block.UPDATE_CLIENTS)
			} else if (energy.batteryLevel.isZero && blockState.getValue(WorkerState.SEMI_WORKER_STATE) != WorkerState.IDLE) {
				level?.setBlock(blockPos, blockState.setValue(WorkerState.SEMI_WORKER_STATE, WorkerState.IDLE), Block.UPDATE_CLIENTS)
			}
		}

		batteryChargeLoop()

		if (isBlockedByRedstone) return
		val level = level ?: return
		val x = blockPos.x.toDouble()
		val y = blockPos.y.toDouble()
		val z = blockPos.z.toDouble()

		for (ent in level.getEntitiesOfClass(ServerPlayer::class.java, AABB(x, y, z, x + 1.0, y + 2.0, z + 1.0))) {
			ent.getCapability(MatteryCapability.MATTERY_PLAYER).ifPresentK {
				if (!it.isAndroid)
					return@ifPresentK

				val missing = it.missingPower

				if (missing > ImpreciseFraction.ZERO) {
					val extract = energy.extractEnergyInner(missing, true)

					if (extract > ImpreciseFraction.ZERO) {
						val received = it.receiveEnergyOuter(extract, false)
						energy.extractEnergyInner(received, false)
					}
				}
			}
		}
	}

	companion object {
		private val MACHINE_NAME = TranslatableComponent("block.overdrive_that_matters.android_station")
		val STORAGE = ImpreciseFraction(100_000)
		val MAX_IO = ImpreciseFraction.valueOf(250)
		val ENERGY_PER_OPERATION = ImpreciseFraction.valueOf(2048)
		val ENERGY_PER_RESEARCH = ImpreciseFraction.valueOf(16384)
	}
}
