package ru.dbotthepony.mc.otm.client.screen.panels

import com.mojang.blaze3d.systems.RenderSystem
import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.client.gui.narration.NarratableEntry
import net.minecraft.client.gui.narration.NarratableEntry.NarrationPriority
import net.minecraft.client.gui.narration.NarrationElementOutput
import net.minecraft.network.chat.Component
import org.lwjgl.opengl.GL30
import ru.dbotthepony.mc.otm.client.playGuiClickSound
import ru.dbotthepony.mc.otm.client.render.*
import ru.dbotthepony.mc.otm.client.screen.MatteryScreen

open class FramePanel(
	screen: MatteryScreen<*>,
	parent: EditablePanel?,
	x: Float,
	y: Float,
	width: Float,
	height: Float,
	protected var title: Component
) : EditablePanel(screen, parent, x, y, width, height), NarratableEntry {
	constructor(screen: MatteryScreen<*>, width: Float, height: Float, title: Component) : this(screen, null, 0f, 0f, width, height, title)

	open inner class Tab(
		var onOpen: Runnable? = null,
		var onClose: Runnable? = null,
		var activeIcon: SkinElement? = null,
		var inactiveIcon: SkinElement? = null,
	) : EditablePanel(this@FramePanel.screen, this@FramePanel, 0f, 0f, 28f, 28f) {
		var isActive = tabs.isEmpty()
		var initial = false

		init {
			tabs.add(this)
		}

		fun showHidePanels(input: List<EditablePanel>) {
			onOpen = Runnable {
				for (child in input) {
					child.visible = true
				}
			}

			onClose = Runnable {
				for (child in input) {
					child.visible = false
				}
			}
		}

		override fun innerRender(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
			val width: Float
			val height: Float

			if (isActive) {
				width = TAB_WIDTH_ACTIVE
				height = TAB_HEIGHT_ACTIVE
				RECTANGLE.middle.render(stack, 2f, 2f, width - 4, height - 2)
			} else {
				width = TAB_WIDTH
				height = TAB_HEIGHT
				TAB_BACKGROUND.render(stack, 2f, 2f, width - 4, height - 2)
			}

			RECTANGLE.top.renderW(stack, 3f, 0f, width - 6)
			RECTANGLE.left.renderH(stack, 0f, 3f, (height - if (isActive) if (initial) 2 else 4 else 3))

			RECTANGLE.right.renderH(stack, width - RECTANGLE.right.w, 3f, (height - if (isActive) 4 else 3))

			RECTANGLE.topLeft.render(stack, 0f, 0f)
			RECTANGLE.topRight.render(stack, width - RECTANGLE.topRight.w, 0f)

			if (isActive) {
				if (!initial) {
					TAB_LEFT_CONNECTION.render(stack, 0f, height - TAB_LEFT_CONNECTION.h - 1)
				}

				TAB_RIGHT_CONNECTION.render(
					stack,
					width - TAB_RIGHT_CONNECTION.w,
					height - TAB_LEFT_CONNECTION.h - 1
				)

				val skinActive = activeIcon ?: return
				skinActive.render(stack, TAB_WIDTH_ACTIVE / 2f - skinActive.w / 2f, TAB_HEIGHT_ACTIVE / 2f - skinActive.h / 2f)
			} else {
				val skinInactive = inactiveIcon ?: return
				skinInactive.render(stack, width / 2f - skinInactive.w / 2f, TAB_HEIGHT_ACTIVE / 2f - skinInactive.h / 2f)
			}
		}

		protected open fun onOpen() {
			onOpen?.run()
		}

		protected open fun onClose() {
			onClose?.run()
		}

		override fun mouseClickedInner(x: Double, y: Double, button: Int): Boolean {
			if (isActive) {
				return true
			}

			for (tab in tabs) {
				if (tab !== this) {
					if (tab.isActive) {
						tab.onClose()
						tab.isActive = false
					}
				}
			}

			isActive = true
			onOpen()
			playGuiClickSound()

			return true
		}
	}

	protected val tabs = ArrayList<Tab>()

	override fun performLayout() {
		for ((i, tab) in tabs.withIndex()) {
			tab.setPos(i * TAB_WIDTH, -TAB_HEIGHT)
			tab.initial = i == 0
		}

		super.performLayout()
	}

	protected var dragging = false

	init {
		setDockPadding(PADDING, PADDING_TOP, PADDING, PADDING)
	}

	override fun mouseClickedInner(x: Double, y: Double, button: Int): Boolean {
		val (_, y1) = screenToLocal(x, y)

		if (parent == null && y1 >= 0 && y1 <= 12) {
			trapMouseInput = true
			dragging = true
			return true
		}

		return super.mouseClickedInner(x, y, button)
	}

	override fun mouseReleasedInner(x: Double, y: Double, button: Int): Boolean {
		if (dragging) {
			trapMouseInput = false
			dragging = false
			return true
		}

		return super.mouseReleasedInner(x, y, button)
	}

	override fun mouseDraggedInner(x: Double, y: Double, button: Int, xDelta: Double, yDelta: Double): Boolean {
		if (dragging) {
			this.x += xDelta.toFloat()
			this.y += yDelta.toFloat()
		}

		return super.mouseDraggedInner(x, y, button, xDelta, yDelta)
	}

	override fun innerRender(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
		RECTANGLE.render(stack, width = width, height = height)

		// title
		RenderSystem.depthFunc(GL30.GL_ALWAYS)
		screen.font.draw(stack, title, 8f, 5f, 4210752)
		RenderSystem.depthFunc(GL30.GL_ALWAYS)
	}

	override fun narrationPriority(): NarrationPriority {
		return NarrationPriority.NONE
	}

	override fun updateNarration(narrationElementOutput: NarrationElementOutput) {}

	fun toScreenCenter() {
		setPos((screen.width / 2 - (width / 2).toInt()).toFloat(), (screen.height / 2 - (height / 2).toInt()).toFloat())
	}

	companion object {
		fun padded(
			screen: MatteryScreen<*>,
			parent: EditablePanel?,
			x: Float,
			y: Float,
			width: Float,
			height: Float,
			title: Component
		) = FramePanel(screen, parent, x, y, width + PADDING * 2, height + PADDING_TOP + PADDING, title)

		fun padded(
			screen: MatteryScreen<*>,
			parent: EditablePanel?,
			width: Float,
			height: Float,
			title: Component
		) = FramePanel(screen, parent, 0f, 0f, width + PADDING * 2, height + PADDING_TOP + PADDING, title)

		const val PADDING = 8f
		const val PADDING_TOP = 14f

		val RECTANGLE = StretchingRectangleElement(
			topLeft = WidgetLocation.WIDGETS.element(x = 18f, y = 0f, w = 6f, h = 6f),
			topRight = WidgetLocation.WIDGETS.element(x = 24f, y = 0f, w = 6f, h = 6f),
			left = WidgetLocation.WIDGETS.element(x = 18f, y = 4f, w = 3f, h = 5f),
			right = WidgetLocation.WIDGETS.element(x = 25f, y = 3f, w = 5f, h = 5f),
			top = WidgetLocation.WIDGETS.element(x = 22f, y = 0f, w = 5f, h = 3f),
			bottomLeft = WidgetLocation.WIDGETS.element(x = 18f, y = 6f, w = 6f, h = 6f),
			bottomRight = WidgetLocation.WIDGETS.element(x = 24f, y = 6f, w = 6f, h = 6f),
			bottom = WidgetLocation.WIDGETS.element(x = 21f, y = 9f, w = 5f, h = 3f),
			middle = WidgetLocation.WIDGETS.element(x = 30f, y = 12f, w = 6f, h = 6f),
			padding = DockProperty(-3f, -3f, -3f, -3f)
		)

		val TAB_RIGHT_CONNECTION 	= WidgetLocation.WIDGETS.element(x = 30f, y = 0f, w = 3f, h = 5f)
		val TAB_LEFT_CONNECTION 	= WidgetLocation.WIDGETS.element(x = 33f, y = 0f, w = 3f, h = 5f)
		val TAB_BACKGROUND 			= WidgetLocation.WIDGETS.element(x = 30f, y = 6f, w = 6f, h = 6f)

		const val TAB_HEIGHT = 28f
		const val TAB_WIDTH = 28f
		const val TAB_HEIGHT_ACTIVE = 32f
		const val TAB_WIDTH_ACTIVE = 28f
	}
}
